package cc.javastudio.raw;

import java.io.*;

public class FileHandler implements Closeable {
    private final int INT_LENGTH = 4;
    private final int BYTE_LENGTH = 1;
    private RandomAccessFile dbFile;

    public FileHandler(String dbFileName) throws FileNotFoundException {
        dbFile = new RandomAccessFile(dbFileName, "rw");
    }

    public void add(String name,
                    int age,
                    String address,
                    String licensePlate,
                    String description) throws IOException {
        //position pointer at the end of file

        long bytePosition = dbFile.length();
        Index.getInstance().add(bytePosition);
        dbFile.seek(bytePosition);
        /*
        calculate the total length of the record
        to put a pointer at the beginning of the record how long the record is
        so you can find the different fields in a stream of bytes
         */

        int recordLength =  INT_LENGTH + name.getBytes().length +
                            INT_LENGTH +
                            INT_LENGTH + address.getBytes().length +
                            INT_LENGTH + licensePlate.getBytes().length +
                            INT_LENGTH + description.getBytes().length;

        /*
        use getBytes() method to ensure special characters like ~ get calculated as well
        (they get stored as 2 bytes instead of one)
         */

        dbFile.writeBoolean(false);
        dbFile.writeInt(recordLength);
        dbFile.writeInt(name.getBytes().length);
        dbFile.write(name.getBytes());
        dbFile.writeInt(age);
        dbFile.writeInt(address.getBytes().length);
        dbFile.write(address.getBytes());
        dbFile.writeInt(licensePlate.getBytes().length);
        dbFile.write(licensePlate.getBytes());
        dbFile.writeInt(description.getBytes().length);
        dbFile.write(description.getBytes());
    }

    public Person readRow(long rowNumber) throws IOException {
        long bytePosition = Index.getInstance().getBytePosition(rowNumber);
        if (bytePosition == -1){
            return null;
        }

        byte[] rawData = readRawRow(bytePosition);
        DataInputStream stream = new DataInputStream(new ByteArrayInputStream(rawData));

        Person person = new Person();
        int stringLength = stream.readInt();
        byte[] nameData = new byte[stringLength];
        stream.read(nameData);
        person.name = new String(nameData);

        person.age = stream.readInt();

        byte[] addressData = new byte[stream.readInt()];
        stream.read(addressData);
        person.address = new String(addressData);

        byte[] plateData = new byte[stream.readInt()];
        stream.read(plateData);
        person.licensePlate = new String(plateData);

        byte[] descriptionData = new byte[stream.readInt()];
        stream.read(descriptionData);
        person.description = new String(descriptionData);

        return person;
    }

    private byte[] readRawRow(long bytePosition) throws IOException {
        dbFile.seek(bytePosition);
        byte[] rawData;

        if (dbFile.readBoolean()){
            rawData =  new byte[0];
        } else {
            int recordLength = dbFile.readInt();
            rawData = new byte[recordLength];
            dbFile.read(rawData);
        }
        return rawData;
    }
    /*
    loadIndex method creates the index contents at the startup of program
    if file doesnt exist, does nothing
    if file already exists, reads every recordlength and moves byteposition to end
     */
    public void loadIndex() throws IOException {
        if (dbFile.length()==0){
            return;
        }
        long bytePos = 0;
        while (bytePos < dbFile.length()){
            dbFile.seek(bytePos);
            boolean isDeleted = dbFile.readBoolean();
            if (!isDeleted){
                Index.getInstance().add(bytePos);
                bytePos += BYTE_LENGTH;
                int recordLength = dbFile.readInt();
                bytePos += INT_LENGTH + recordLength;

            }
        }
    }

    public void close() throws IOException {
        dbFile.close();
    }
}

